<?php

namespace Litepci\Project\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litepci\Project\Http\Requests\MilestoneRequest;
use Litepci\Project\Interfaces\MilestoneRepositoryInterface;
use Litepci\Project\Models\Milestone;

/**
 * Resource controller class for milestone.
 */
class MilestoneResourceController extends BaseController
{

    /**
     * Initialize milestone resource controller.
     *
     * @param type MilestoneRepositoryInterface $milestone
     *
     * @return null
     */
    public function __construct(MilestoneRepositoryInterface $milestone)
    {
        parent::__construct();
        $this->repository = $milestone;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litepci\Project\Repositories\Criteria\MilestoneResourceCriteria::class);
    }

    /**
     * Display a list of milestone.
     *
     * @return Response
     */
    public function index(MilestoneRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litepci\Project\Repositories\Presenter\MilestonePresenter::class)
                ->$function();
        }

        $milestones = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('project::milestone.names'))
            ->view('project::milestone.index', true)
            ->data(compact('milestones', 'view'))
            ->output();
    }

    /**
     * Display milestone.
     *
     * @param Request $request
     * @param Model   $milestone
     *
     * @return Response
     */
    public function show(MilestoneRequest $request, Milestone $milestone)
    {

        if ($milestone->exists) {
            $view = 'project::milestone.show';
        } else {
            $view = 'project::milestone.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('project::milestone.name'))
            ->data(compact('milestone'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new milestone.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(MilestoneRequest $request)
    {

        $milestone = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('project::milestone.name')) 
            ->view('project::milestone.create', true) 
            ->data(compact('milestone'))
            ->output();
    }

    /**
     * Create new milestone.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(MilestoneRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $milestone                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('project::milestone.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('project/milestone/' . $milestone->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/project/milestone'))
                ->redirect();
        }

    }

    /**
     * Show milestone for editing.
     *
     * @param Request $request
     * @param Model   $milestone
     *
     * @return Response
     */
    public function edit(MilestoneRequest $request, Milestone $milestone)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('project::milestone.name'))
            ->view('project::milestone.edit', true)
            ->data(compact('milestone'))
            ->output();
    }

    /**
     * Update the milestone.
     *
     * @param Request $request
     * @param Model   $milestone
     *
     * @return Response
     */
    public function update(MilestoneRequest $request, Milestone $milestone)
    {
        try {
            $attributes = $request->all();

            $milestone->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('project::milestone.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('project/milestone/' . $milestone->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('project/milestone/' . $milestone->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the milestone.
     *
     * @param Model   $milestone
     *
     * @return Response
     */
    public function destroy(MilestoneRequest $request, Milestone $milestone)
    {
        try {

            $milestone->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('project::milestone.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('project/milestone/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('project/milestone/' . $milestone->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple milestone.
     *
     * @param Model   $milestone
     *
     * @return Response
     */
    public function delete(MilestoneRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('project::milestone.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('project/milestone'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/project/milestone'))
                ->redirect();
        }

    }

    /**
     * Restore deleted milestones.
     *
     * @param Model   $milestone
     *
     * @return Response
     */
    public function restore(MilestoneRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('project::milestone.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/project/milestone'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/project/milestone/'))
                ->redirect();
        }

    }

}
